/*
 * This file is part of piggy
 * Copyright (c) 2005 Gianni Tedesco <gianni@scaramanga.co.uk>
 * Released under the terms of the GNU GPL version 2
*/

#include <piggy.h>
#include <nbio.h>
#include <fdctl.h>

#include <errno.h> /* errno */
#include <sys/socket.h>  /* inet_aton(3) */
#include <netinet/in.h> /* inet_aton(3)/htons(3) */
#include <arpa/inet.h> /* inet_aton(3) */

static const char *cmd;
static struct iothread iothread;

int main(int argc, char **argv)
{
	struct list_head listeners = LIST_HEAD_INIT(listeners);
	struct nbio *io, *tmp;
	const char *plugin = NULL;
	struct proxy_args arg;
	struct in_addr in;

	cmd = argv[0];

	sig_pipeignore();

	if ( !ssl_proxy_init() )
		return EXIT_FAILURE;

	if ( !inet_aton("66.212.233.235", &in) )
		return EXIT_FAILURE;
	arg.addr = in.s_addr;
	arg.port = htons(444);

	/* TODO: getopt */
	io = listener_new(0, 1234, ssl_proxy_new, &arg);
	if ( io )
		list_add(&io->list, &listeners);

	if ( !nbio_init(&iothread, plugin) )
		return EXIT_FAILURE;

	list_for_each_entry_safe(io, tmp, &listeners, list) {
		list_del(&io->list);
		listener_add(&iothread, io);
	}

	printf("%s: Using %s eventloop plugin\n",
		cmd, iothread.plugin->name);

	do {
		nbio_pump(&iothread);
	}while ( !list_empty(&iothread.active) );
	

	nbio_fini(&iothread);

	return EXIT_SUCCESS;
}
